

class Participant:

    def __init__(self):
        self.code = ''
        self.name = ''
        self.role = ''
        self.language = ''
        self.corpus = ''
        self.age = ''
        self.sex = ''
        self.group = ''
        self.ses = ''
        self.role = ''
        self.education = ''
        self.custom = ''
        self.birth_date = ''
