

class Record:

    def __init__(self):
        self.uid = -1
        self.participant_code = ''
        self.utterance = ''
        self.start_time = ''
        self.end_time = ''
        self.dependent_tiers = {}
